﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectDumpTruck.Drawnings;
using ProjectDumpTruck.Exceptions;

namespace ProjectDumpTruck.CollectionGenericObject;

public abstract class AbstractCompany 
{
    /// <summary>
    /// Размер места (ширина)
    /// </summary>
    protected readonly int _placeSizeWidth = 310;

    /// <summary>
    /// Размер места (высота)
    /// </summary>
    protected readonly int _placeSizeHeight = 125;

    /// <summary>
    /// Ширина окна
    /// </summary>
    protected readonly int _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    protected readonly int _pictureHeight;

    /// <summary>
    /// Коллекция грузовиков
    /// </summary>
    protected ICollectionGenericObject<DrawningTruck>? _collection = null;
    
    /// <summary>
    /// Вычисление максимального количества элементов, который можно разместить в окне
    /// </summary>
    private int GetMaxCount => _pictureWidth * _pictureHeight / (_placeSizeWidth * _placeSizeHeight);
    
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="picWidth">Ширина окна</param>
    /// <param name="picHeight">Высота окна</param>
    /// <param name="collection">Коллекция грузовиков</param>
    public AbstractCompany(int picWidth, int picHeight,
    ICollectionGenericObject<DrawningTruck> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = collection;
        _collection.MaxCount = GetMaxCount;
    }
    /// <summary>
    /// Перегрузка оператора сложения для класса
    /// </summary>
    /// <param name="company">Компания</param>
    /// <param name="truck">Добавляемый объект</param>
    /// <returns></returns>
    public static int operator +(AbstractCompany company, DrawningTruck truck)
    {
        try
        {
            return company._collection.Insert(truck, 0, new DrawningTruckEqutables());
        }
        catch (ObjectAlreadyInCollectionException)
        {
            return -1;
        }
        catch (CollectionOverflowException)
        {
            return -1;
        }
    }

    /// <summary>
    /// Перегрузка оператора удаления для класса
    /// </summary>
    /// <param name="company">Компания</param>
    /// <param name="position">Номер удаляемого объекта</param>
    /// <returns></returns>
    public static DrawningTruck? operator -(AbstractCompany company, int position)
    {
        return company._collection?.Remove(position);
    }

    /// <summary>
    /// Получение случайного объекта из коллекции
    /// </summary>
    /// <returns></returns>
    public DrawningTruck? GetRandomObject()
    {
        Random rnd = new();
        try
        {
            return _collection?.Get(rnd.Next(GetMaxCount));
        }
        catch (ObjectNotFoundException)
        {
            return null;
        }
    }

    /// <summary>
    /// Вывод всей коллекции
    /// </summary>
    /// <returns></returns>
    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackground(graphics);

        SetObjectsPosition();
        for (int i = 0; i < (_collection?.Count ?? 0); ++i)
        {
            try
            {
                DrawningTruck? obj = _collection?.Get(i);
                obj?.DrawTransport(graphics);
            }
            catch (ObjectNotFoundException)
            {
                continue;
            }
        }
        return bitmap;
    }

    /// <summary>
    /// Вывод заднего фона
    /// </summary>
    /// <param name="g"></param>
    protected abstract void DrawBackground(Graphics g);

    /// <summary>
    /// Расстановка объектов
    /// </summary>
    protected abstract void SetObjectsPosition();

    /// <summary>
    /// Сортировка
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    public void Sort(IComparer<DrawningTruck?> comparer) => _collection?.CollectionSort(comparer);


}
