﻿using ProjectDumpTruck.Drawnings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectDumpTruck.CollectionGenericObject;

public interface ICollectionGenericObject<T>
    where T : class
{

    /// <summary>
    /// Кол-во объектов в коллекции
    /// </summary>
    int Count { get; }

    /// <summary>
    /// Установка макс кол-ва элементов
    /// </summary>
    int MaxCount { get;  set; }

    /// <summary>
    /// Добавление объекта в коллекцию
    /// </summary>
    /// <param name="obj">Добавляемый объект</param>
    /// <returns>true - вставка прошла удачно, false - вставка не удалась</returns>
    int Insert(T obj, IEqualityComparer<DrawningTruck?>? comparer = null);

    /// <summary>
    /// Добавление объекта на конкретную позиуцию
    /// </summary>
    /// <param name="obj">Добавляемый объект</param
    /// <param name="position">Позиция</param>
    /// <returns>true - вставка прошла успешна, false - вставка не удалась</returns>
    int Insert(T obj, int position, IEqualityComparer<DrawningTruck?>? comparer = null);

    /// <summary>
    /// Удаление объекта из коллекции с конкретной позиции
    /// </summary>
    /// <param name="position">Позиция</param>
    /// <returns>true - удаление прошло удачно, false - удаление не удалось</returns>
    T? Remove(int position);

    /// <summary>
    /// Получение объекта по позиции
    /// </summary>
    /// <param name="position">Позиция</param>
    /// <returns>Объект</returns>
    T? Get(int position);

    /// <summary>
    /// Получение типа коллекции
    /// </summary>
    CollectionType GetCollectionType { get; }

    /// <summary>
    /// Получение объектов из коллекции по одному
    /// </summary>
    /// <returns></returns>
    IEnumerable<T> GetItems();

    /// <summary>
    /// Сортировка коллекции
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    void CollectionSort(IComparer<T?> comparer);
}
