﻿    using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Metadata.Ecma335;
using System.Text;
using System.Threading.Tasks;
using ProjectDumpTruck.Drawnings;
using ProjectDumpTruck.Exceptions;

namespace ProjectDumpTruck.CollectionGenericObject;

public class MassiveGenericObjects<T> : ICollectionGenericObject<T>
    where T : class
{

    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }
        if (_collection[position] == null)
        {
            throw new ObjectNotFoundException(position);
        }
        // TODO проверка позиции
        if (position < 0 || position >= Count)
        {
            return null;
        }
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<DrawningTruck?>? comparer = null)
    {
        for (int i = 0; i < Count; i++)
        {
            if (comparer.Equals((_collection[i] as DrawningTruck), (obj as DrawningTruck)))
            {
                throw new ObjectAlreadyInCollectionException(i);
            }
        }

        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position, IEqualityComparer<DrawningTruck?>? comparer = null)
    {
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }

        for (int i = 0; i < Count; i++)
        {
            if (comparer.Equals((_collection[i] as DrawningTruck), (obj as DrawningTruck)))
            {
                throw new ObjectAlreadyInCollectionException(i);
            }
        }

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }

        for (int i = position + 1; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        for (int i = position - 1; i >= 0; i--)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        throw new CollectionOverflowException(Count);
    }
    public T? Remove(int position)
    {
        //  проверка позиции
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }

        if (_collection[position] == null)
        {
            throw new ObjectNotFoundException(position);
        }

        T? temp = _collection[position];
        _collection[position] = null;
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; i++) yield return _collection[i];
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}